const EXT_LABEL = 'ext-ff';
const INSTALL_PARAMS = {
  source: 'organic',
  campaign: 'none',
  date: '2000-01-01',
  live: false,
};
const UNINSTALL_URL_BASE = 'https://add.startpage.com/uninstall/';

/*
 * Capture OnInstalled event.
 * Communicate it to content script.
 */
browser.runtime.onInstalled.addListener(e => {
  browser.tabs.query({
    currentWindow: true,
    active: true,
  }).then(sendMessageToTabs);
  launchSuccess(e.reason);
});

browser.webRequest.onBeforeSendHeaders.addListener(
  augmentSearch,
  {
    urls: [
      'https://*.startpage.com/*',
    ],
  },
  ['blocking', 'requestHeaders']
);

browser.runtime.onMessage.addListener(msg => {
  if (msg.event !== 'spcontentpl') return;
  browser.storage.local.get(INSTALL_PARAMS)
    .then(
      data => {
        if (data.live) return data;
        const args = Object.keys(INSTALL_PARAMS).reduce(
          (a, p) => (a[p] = msg[p], a), {}
        );
        if (Object.values(args).every(a => a)) {
          browser.storage.local.set(args);
        }
        const u = buildUrl(UNINSTALL_URL_BASE, args)
        browser.runtime.setUninstallURL(u);
        return data;
      },
      () => browser.runtime.setUninstallURL(UNINSTALL_URL_BASE)
    );
});

const RETRY_PERIOD = 200;
const MAX_TRIES = (60 * 1000) / RETRY_PERIOD;

function launchSuccess(reason, tries=1) {
  if (reason !== 'install' || tries > MAX_TRIES) return;

  browser.search.get().then(engines => {
    const spDefault = engines.some(
      e => e.name.startsWith('Startpage.com') && e.isDefault
    )
    if (spDefault) {
      browser.tabs.create({url: 'https://add.startpage.com/success/'});
    } else {
      setTimeout(
        launchSuccess, RETRY_PERIOD, reason, ++tries
      );
    }
  });
}

function sendMessageToTabs(tabs) {
  for (const tab of tabs) {
    browser.tabs.sendMessage(
      tab.id,
      {msg: 'ATB-Plugin-Installed'}
    );
  }
  return tabs;
}

function augmentSearch(e) {
  return browser.storage.local.get(INSTALL_PARAMS)
    .then(data => {
      const extensionHeaders = [
        {name: 'Startpage-Extension', value: EXT_LABEL},
      ];
      if (e.url.includes(`&pl=${EXT_LABEL}&`)) {
        extensionHeaders.push(
          {name: 'Startpage-Extension-Campaign', value: data.campaign}
        );
        extensionHeaders.push(
          {name: 'Startpage-Extension-Date', value: data.date}
        );
        extensionHeaders.push(
          {name: 'Startpage-Extension-Source', value: data.source}
        );
      }
      const newHeaders = e.requestHeaders.concat(extensionHeaders);
      return {requestHeaders: newHeaders};
    });
}

function buildUrl(base, data) {
  const u = new URL(base);
  u.searchParams.set('campaign', data.campaign);
  u.searchParams.set('date', data.date);
  u.searchParams.set('source', data.source);
  u.searchParams.set('extVersion', browser.runtime.getManifest().version)
  return u.toString();
}

// TODO: handle localization of urls later
function getLangCode() {
  return browser.i18n.getUILanguage().split('-')[0] || 'en';
}
